noggin(1)

# NAME

*noggin* - face and expression tracking

# SYNOPSIS

*noggin* [*--cam=*_DEVICE_] [*--show*] [*--verbose*] _SUBSCRIPTIONS_...

# DESCRIPTION

noggin is a command-line utility for realtime face and expression tracking. It
continuously prints tracking results based on it's arguments. It only tracks
one human face.

# OPTIONS

*--cam=*_DEVICE_
	Specify the capture device. (e.g. *0* or */dev/video0*)

*-h*, *--help*
	Print help and exit.

*--show*
	Display a window visualizing the tracking.

*--verbose*
	Print log messages.

*--version*
	Print the version and exit.

# ARGUMENTS

Each argument specifies another thing to output. The supported arguments are:

*brow*
	Print whether the person's eyebrows are up, down or neutral.

*position*
	Print the person's head position.

*rotation* [*deadzone=*_MAGNITUDE_] [*lowpass=*_PERCENTAGE_]
	Print the person's head rotation around the x, y and z axes. The
	optional deadzone sets a threshold to ignore for the x and y axes. The
	optional lowpass (between 0.0 and 1.0), when set low, softens the
	change in the results.

*rotation-change* [*lowpass=*_PERCENTAGE_]
	Print the change in the person's head rotation around the x, y and
	z axes. The optional lowpass (between 0.0 and 1.0), when set low,
	softens the change in the results.

# MODELS

Noggin is usually packaged with an alternative landmarking model which is
lighter weight but too rigid for accurate expression recognition. You can
specify the model's path with the *NOGGIN_MODEL* environment variable,
and find some other alternatives at: *https://git.sr.ht/~geb/noggin-models*

# EXAMPLES

Print head position and rotation results while visualizing the tracking:

	$ noggin position 'rotation lowpass=0.1' --show

Simply visualize the tracking while printing log messages:

	$ noggin --show --verbose

Move the mouse cursor with rotation of your head (using the *dotool* command):

```
$ awk -f /dev/stdin << 'EOF'
BEGIN {
	while ("noggin 'rotation-change lowpass=0.1'" | getline == 1) {
		print "mousemove "($3 * -30)" "($2 * -30) | "dotool"
	}
}
EOF
```

# AUTHOR

John Gebbie
