#!/bin/sh
# install.sh [DESTDIR] [BINDIR]
: "${NOGGIN_VERSION=$(git describe --long --abbrev=12 --tags --dirty 2>/dev/null || echo 0.1)}"
: "${NOGGIN_MODEL_DIR=/usr/share/noggin-models}"
: "${NOGGIN_MODEL_PACKAGE=}"
: "${NOGGIN_EXAMPLES_DIR=$1/usr/share/examples/noggin}"
: "${NOGGIN_MANPAGE_DIR=$1/usr/share/man/man1}"

if ! [ "$NOGGIN_SKIP_CHECKS" ]; then
	ok=1
	! command -v gcc >/dev/null && echo 'you need gcc' && unset ok
	! command -v go >/dev/null && echo 'you need go (a.k.a golang)' && unset ok
	! command -v scdoc >/dev/null && echo 'you need scdoc' && unset ok
	[ "$ok" ] || exit
fi

if ! [ "$NOGGIN_SKIP_BINARY" ]; then
	go build -ldflags "-X 'main.Version=$NOGGIN_VERSION'
		-X 'main.ModelDir=$NOGGIN_MODEL_DIR'
		-X 'main.ModelPackage=$NOGGIN_MODEL_PACKAGE'" || exit
	echo 'Built successfully.'
fi

if [ "$NOGGIN_SKIP_INSTALL" ]; then
	exit
fi

if ! [ "$NOGGIN_SKIP_BINARY" ]; then
	mkdir -p "$1/${2:-usr/local/bin}" || exit
	cp noggin "$1/${2:-usr/local/bin}" || exit
fi

mkdir -p "$1/$NOGGIN_MODEL_DIR" || exit
cp models/mnv3_detection_opt.onnx "$1/$NOGGIN_MODEL_DIR" || exit
if ! [ "$NOGGIN_SKIP_MODELS" ]; then
	cp models/lm_model0_opt.onnx "$1/$NOGGIN_MODEL_DIR" || exit
	cp models/lm_model3_opt.onnx "$1/$NOGGIN_MODEL_DIR" || exit
fi

if ! [ "$NOGGIN_SKIP_EXAMPLES" ]; then
	mkdir -p "$NOGGIN_EXAMPLES_DIR" || exit
	cp -r examples/* "$NOGGIN_EXAMPLES_DIR" || exit
fi

mkdir -p "$NOGGIN_MANPAGE_DIR" || exit
scdoc < doc/noggin.1.scd > "$NOGGIN_MANPAGE_DIR/noggin.1" || exit

echo 'Installed successfully.'
